from taranis_base_bot import create_app
from taranis_base_bot.config import get_common_settings


def test_default_modelinfo_uses_hf_when_enabled(monkeypatch):
    import taranis_base_bot as pkg

    class DummyModel:
        model_name = "hf-model"
        modelinfo = {"should_not_be_used": True}

        def predict(self, **kw):
            return kw

    def fake_get_model(_cfg):
        return DummyModel()

    def fake_get_hf_modelinfo(name):
        assert name == "hf-model"
        return {"source": "hf", "name": name}

    monkeypatch.setattr(pkg, "get_model", fake_get_model)
    monkeypatch.setattr(pkg, "get_hf_modelinfo", fake_get_hf_modelinfo)

    cfg = get_common_settings()
    cfg.HF_MODEL_INFO = True

    app = create_app(
        name="svc-hf",
        config=cfg,
        request_parser=lambda x: x,
        method_decorators=[],
    )
    with app.test_client() as c:
        r = c.get("/modelinfo")
        assert r.status_code == 200
        assert r.get_json() == {"source": "hf", "name": "hf-model"}


def test_default_modelinfo_uses_local_when_disabled(monkeypatch):
    import taranis_base_bot as pkg

    class DummyModel:
        model_name = "hf-model"
        modelinfo = {"source": "local"}

        def predict(self, **kw):
            return kw

    def fake_get_model(_cfg):
        return DummyModel()

    def fake_get_hf_modelinfo(_name):
        raise AssertionError("HF modelinfo should not be called")

    monkeypatch.setattr(pkg, "get_model", fake_get_model)
    monkeypatch.setattr(pkg, "get_hf_modelinfo", fake_get_hf_modelinfo)

    cfg = get_common_settings()
    cfg.HF_MODEL_INFO = False

    app = create_app(
        name="svc-local",
        config=cfg,
        request_parser=lambda x: x,
        method_decorators=[],
    )
    with app.test_client() as c:
        r = c.get("/modelinfo")
        assert r.status_code == 200
        assert r.get_json() == {"source": "local"}


def test_default_request_parser_uses_config_payload_key(monkeypatch):
    import taranis_base_bot as pkg

    class DummyModel:
        model_name = "m"
        modelinfo = {"m": True}

        def predict(self, **kw):
            return kw

    monkeypatch.setattr(pkg, "get_model", lambda _cfg: DummyModel())

    cfg = get_common_settings()
    cfg.PAYLOAD_KEY = "test"

    app = create_app(
        name="svc-parser",
        config=cfg,
        method_decorators=[],
    )
    with app.test_client() as c:
        r = c.post("/", json={"test": "test val"})
        assert r.status_code == 200
        assert r.get_json() == {"test": "test val"}


def test_url_prefix_routes(monkeypatch):
    import taranis_base_bot as pkg

    class DummyModel:
        model_name = "m"
        modelinfo = {"mi": True}

        def predict(self, **kw):
            return kw

    monkeypatch.setattr(pkg, "get_model", lambda _cfg: DummyModel())

    cfg = get_common_settings()
    cfg.PAYLOAD_KEY = "text"
    app = create_app(
        name="svc-prefixed",
        config=cfg,
        url_prefix="/v1",
        method_decorators=[],
    )

    with app.test_client() as c:
        r1 = c.get("/v1/health")
        assert r1.status_code == 200
        assert r1.get_json() == {"status": "ok"}

        r2 = c.get("/v1/modelinfo")
        assert r2.status_code == 200
        assert r2.get_json() == {"mi": True}

        r3 = c.post("/v1/", json={"text": "hello"})
        assert r3.status_code == 200
        assert r3.get_json() == {"text": "hello"}



import os
import pytest

from dotenv import load_dotenv
from taranis_base_bot.config import get_common_settings
from taranis_base_bot.decorators import api_key_required
from taranis_base_bot.misc import get_hf_modelinfo


base_dir = os.path.dirname(os.path.abspath(__file__))
env_file = os.path.join(base_dir, ".env.test")
current_path = os.getcwd()

load_dotenv(dotenv_path=env_file, override=True)

Config = get_common_settings()


@pytest.fixture
def client():
    from taranis_base_bot import create_app

    app = create_app(
        name="taranis_base_bot",
        config=Config,
        url_prefix="/",
        predict_fn=lambda **kwargs: kwargs,
        modelinfo_fn=lambda: {"model": "test"},
        request_parser=lambda x: x,
        method_decorators=[],
    )
    app.config["TESTING"] = True
    with app.test_client() as client:
        yield client


@pytest.fixture
def client_with_request_parser():
    from taranis_base_bot import create_app

    def request_parser(data: dict[str, str]) -> dict[str, str]:
        if "text" not in data:
            raise ValueError("Data must contain 'text' key")
        return {"text": data["text"]}

    app = create_app(
        name="taranis_base_bot",
        config=Config,
        url_prefix="/",
        predict_fn=lambda **kwargs: kwargs,
        modelinfo_fn=lambda: {"model": "test"},
        request_parser=request_parser,
        method_decorators=[],
    )
    app.config["TESTING"] = True
    with app.test_client() as client:
        yield client


@pytest.fixture
def client_with_predict():
    from taranis_base_bot import create_app

    def predict_func(text: str) -> dict[str, int]:
        if not isinstance(text, str):
            raise ValueError("Input is not a string!")
        return {"len": len(text)}

    app = create_app(
        name="taranis_base_bot",
        config=Config,
        url_prefix="/",
        predict_fn=predict_func,
        modelinfo_fn=lambda: {"model": "test"},
        request_parser=lambda x: x,
        method_decorators=[],
    )
    app.config["TESTING"] = True
    with app.test_client() as client:
        yield client


@pytest.fixture
def client_with_modelinfo_fn():
    from taranis_base_bot import create_app

    app = create_app(
        name="taranis_base_bot",
        config=Config,
        url_prefix="/",
        predict_fn=lambda **kwargs: kwargs,
        modelinfo_fn=lambda: get_hf_modelinfo("test_model"),
        request_parser=lambda x: x,
        method_decorators=[],
    )
    app.config["TESTING"] = True
    with app.test_client() as client:
        yield client


@pytest.fixture
def client_with_api_key(monkeypatch):
    """Client with API key authentication enabled"""
    # Temporarily set API key for testing
    monkeypatch.setenv("API_KEY", "test-api-key-123")

    # Reload config to pick up the new API key
    Config.API_KEY = "test-api-key-123"
    from taranis_base_bot import create_app

    app = create_app(
        name="taranis_base_bot_with_api_key",
        config=Config,
        url_prefix="/",
        predict_fn=lambda **kwargs: kwargs,
        modelinfo_fn=lambda: {"model": "test"},
        request_parser=lambda x: x,
        method_decorators=[api_key_required],
    )
    app.config["TESTING"] = True
    with app.test_client() as client:
        yield client



import sys
import importlib
from types import SimpleNamespace
import textwrap
import pytest

from taranis_base_bot.misc import create_request_parser, get_model, get_hf_modelinfo


def test_create_request_parser_success():
    parser = create_request_parser("text", str)
    data = {"text": "hello"}
    out = parser(data)
    assert out == {"text": "hello"}


def test_create_request_parser_missing_key():
    parser = create_request_parser("text", str)
    with pytest.raises(ValueError) as ei:
        parser({"other": "x"})
    assert str(ei.value) == "Payload does not contain 'text' key!"


def test_create_request_parser_empty_value_string():
    parser = create_request_parser("text", str)
    with pytest.raises(ValueError) as ei:
        parser({"text": ""})
    assert str(ei.value) == "No data provied for 'text' key!"


def test_create_request_parser_wrong_type():
    parser = create_request_parser("text", str)
    with pytest.raises(ValueError) as ei:
        parser({"text": 123})
    assert str(ei.value) == "Data for 'text' is not of type '<class 'str'>'"


def test_create_request_parser_empty_list_also_rejected():
    parser = create_request_parser("items", list)
    with pytest.raises(ValueError) as ei:
        parser({"items": []})
    assert str(ei.value) == "No data provied for 'items' key!"


def _write_module(tmp_path, package_name: str, module_name: str, source: str):
    """Helper to write a package and a module file, returning package path."""
    pkg_dir = tmp_path / package_name
    pkg_dir.mkdir()
    (pkg_dir / "__init__.py").write_text("")
    (pkg_dir / f"{module_name}.py").write_text(textwrap.dedent(source))
    return pkg_dir


def test_get_model_success(tmp_path, monkeypatch):
    """
    get_model should import <PACKAGE_NAME>.<MODEL>, derive the class name from MODEL,
    and return an instance of that class.
    """
    package = "dynpkg"
    module = "toy_model"
    src = """
    class ToyModel:
        def __init__(self):
            self.ok = True
    """
    _ = _write_module(tmp_path, package, module, src)

    monkeypatch.syspath_prepend(str(tmp_path))
    importlib.invalidate_caches()

    cfg = SimpleNamespace(PACKAGE_NAME=package, MODEL=module)

    # Exercise
    model = get_model(cfg)
    assert getattr(model, "ok", False) is True

    for mod in [f"{package}.{module}", package]:
        sys.modules.pop(mod, None)


def test_get_model_missing_module_raises(tmp_path, monkeypatch):
    package = "dynpkg2"
    _write_module(tmp_path, package, "__init__", "")
    monkeypatch.syspath_prepend(str(tmp_path))
    importlib.invalidate_caches()

    cfg = SimpleNamespace(PACKAGE_NAME=package, MODEL="does_not_exist")
    with pytest.raises(ImportError) as ei:
        get_model(cfg)
    assert f"Package {package} has no module named {cfg.MODEL}" in str(ei.value)

    for mod in [package]:
        sys.modules.pop(mod, None)


def test_get_model_missing_class_raises(tmp_path, monkeypatch):
    package = "dynpkg3"
    module = "bad_model"
    src = """
    class AnotherClass:
        pass
    """
    _write_module(tmp_path, package, module, src)
    monkeypatch.syspath_prepend(str(tmp_path))
    importlib.invalidate_caches()

    cfg = SimpleNamespace(PACKAGE_NAME=package, MODEL=module)
    with pytest.raises(ImportError) as ei:
        get_model(cfg)
    assert f"Module {module} has no class named BadModel" in str(ei.value)

    for mod in [f"{package}.{module}", package]:
        sys.modules.pop(mod, None)


def test_get_hf_modelinfo_http_error(requests_mock):
    model_name = "missing-model"
    requests_mock.get(
        f"https://huggingface.co/api/models/{model_name}",
        status_code=404,
        json={"error": "not found"},
    )
    out = get_hf_modelinfo(model_name)
    assert out["model"] == model_name
    assert "error" in out


def test_get_hf_modelinfo_exception_returns_fallback(monkeypatch):
    model_name = "any-model"

    import requests

    def unexpected_error(*a, **k):
        raise RuntimeError("Fail!")

    monkeypatch.setattr(requests, "get", unexpected_error)

    out = get_hf_modelinfo(model_name)
    assert out["model"] == model_name
    assert out["error"] == "Fail!"



from datetime import datetime
from functools import lru_cache
from typing import Dict, Optional

from pydantic import field_validator, ValidationInfo
from pydantic_settings import BaseSettings, SettingsConfigDict


class CommonSettings(BaseSettings):
    model_config = SettingsConfigDict(env_file=".env", env_file_encoding="utf-8", extra="ignore")

    DEBUG: bool = False
    API_KEY: str = ""

    COLORED_LOGS: bool = True
    BUILD_DATE: datetime = datetime.now()
    GIT_INFO: Optional[Dict[str, str]] = None
    CACHE_TYPE: str = "SimpleCache"
    CACHE_DEFAULT_TIMEOUT: int = 300

    MODEL: str = ""
    PACKAGE_NAME = "taranis_bot"
    HF_MODEL_INFO: bool = False
    PAYLOAD_KEY: str = "key"

    @field_validator("API_KEY", mode="before")
    @classmethod
    def check_non_empty_string(cls, v: str, info: ValidationInfo) -> str:
        if not isinstance(v, str) or not v.strip():
            print("API_KEY is not set or empty, disabling API key requirement")
        return v


@lru_cache(maxsize=1)
def get_common_settings() -> CommonSettings:
    return CommonSettings()



import requests
from pydantic_settings import BaseSettings
from typing import Any, Callable
from importlib import import_module
from taranis_base_bot.protocols import Predictor


def create_request_parser(key: str, val_type: type) -> Callable[[dict], dict]:
    def request_parser(data: dict) -> dict[str, Any]:
        if key not in data:
            raise ValueError(f"Payload does not contain '{key}' key!")
        val = data[key]

        if val is None or not val:
            raise ValueError(f"No data provided for '{key}' key!")

        if not isinstance(val, val_type):
            raise ValueError(f"Data for '{key}' is not of type '{val_type}'")

        return data

    return request_parser


def get_model(config: BaseSettings) -> Predictor:
    try:
        module = import_module(f"{config.PACKAGE_NAME}.{config.MODEL}")
    except ModuleNotFoundError as e:
        raise ImportError(
            f"Package {config.PACKAGE_NAME} has no module named {config.MODEL}. Make sure the file {config.MODEL}.py exists"
        ) from e

    class_name = "".join(part.capitalize() for part in config.MODEL.split("_") if part)
    model_class = getattr(module, class_name, None)
    if model_class is None:
        raise ImportError(f"Module {config.MODEL} has no class named {class_name}. Make sure the class exists in {config.MODEL}.py")

    return model_class()


def get_hf_modelinfo(model_name: str) -> dict:
    """
    Fetch model metadata from Hugging Face.
    If anything fails, return a simple fallback.
    """
    try:
        url = f"https://huggingface.co/api/models/{model_name}"
        response = requests.get(url, timeout=10)
        response.raise_for_status()
        return response.json()
    except Exception as e:
        return {"model": model_name, "error": str(e)}



from flask import Flask
from typing import Callable, Any
from taranis_base_bot import blueprint
from taranis_base_bot.misc import get_model, get_hf_modelinfo, create_request_parser
from taranis_base_bot.decorators import api_key_required
from taranis_base_bot.blueprint import JSON
from taranis_base_bot.log import configure_logger
from taranis_base_bot.config import CommonSettings


def create_app(
    name: str,
    config: CommonSettings,
    url_prefix: str = "/",
    predict_fn: Callable[..., Any] | None = None,
    modelinfo_fn: Callable[[], Any] | None = None,
    request_parser: Callable[[JSON], dict[str, Any]] | None = None,
    method_decorators: list[Callable] | None = None,
    setup_logging: bool = True,
) -> Flask:
    app = Flask(__name__)
    app.config.from_object(config)
    app.url_map.strict_slashes = False

    if setup_logging:
        configure_logger(
            module=config.PACKAGE_NAME,
            debug=config.DEBUG,
            colored=config.COLORED_LOGS,
            syslog_address=None,
        )

    model = None
    if predict_fn is None or modelinfo_fn is None:
        model = get_model(config)

    if predict_fn is None:
        predict_fn = model.predict  # type: ignore[assignment]

    if modelinfo_fn is None:

        def default_modelinfo_fn():
            if config.HF_MODEL_INFO:
                return get_hf_modelinfo(model.model_name)  # type: ignore[attr-defined]
            else:
                return model.model_name  # type: ignore[attr-defined]

        modelinfo_fn = default_modelinfo_fn

    if request_parser is None:
        request_parser = create_request_parser(config.PAYLOAD_KEY, str)

    if method_decorators is None:
        method_decorators = [api_key_required]

    bp = blueprint.create_service_blueprint(
        name=name,
        url_prefix=url_prefix,
        predict_fn=predict_fn,
        modelinfo_fn=modelinfo_fn,
        request_parser=request_parser,
        method_decorators=method_decorators,
    )
    app.register_blueprint(bp)
    return app


if __name__ == "__main__":
    raise SystemExit("Cannot run taranis-base-bot directly. Use 'from taranis-base-bot import create_app' instead.")
